import sys
import logging
import logging.config
import io
import yaml


from .logxintrospect import get_nicest_module_name

YAML_CONFIG_PATH = "loggers.yaml"
DEFAULT_FORMAT = (
    "[%(asctime)s] %(levelname)s [%(name)s.%(funcName)s:%(lineno)d] %(message)s"
)


def load_yaml_config(yaml_config_path=YAML_CONFIG_PATH):
    with io.open(yaml_config_path) as f:
        yaml_config_text = f.read()
    yaml_config = yaml.load(yaml_config_text)
    logging.config.dictConfig(yaml_config)


class StdHandler(logging.Handler):
    def emit(self, record):
        msg = self.format(record)
        if record.levelno >= logging.WARNING:
            stream = sys.stderr
        else:
            stream = sys.stdout
        stream.write(msg)
        stream.write("\n")
        self.flush()


def set_basic_config():
    logging.basicConfig(level=logging.DEBUG)


def set_root_level(level="DEBUG"):
    get_logger("").setLevel(getattr(logging, level))


def get_logger(name):
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    return logger


def print_diagram():
    import logging_tree

    logging_tree.printout()


class Log:
    def __init__(self):
        self.handlers = {}

    def set_level(self, level, name=None):
        handler = self.get_handler(name)
        handler.setLevel(getattr(logging, level.upper()))

    def add_std_handler(self, logger_name, level="DEBUG", top_level=True):
        if top_level:
            logger_name = logger_name.split(".")[0]
        level = level.upper()
        logging_level = getattr(logging, level)
        logger = logging.getLogger(logger_name)
        logger.setLevel(logging_level)
        h = StdHandler()
        h.setLevel(logging_level)
        logger.addHandler(h)
        return h

    def current_logger_name(self):
        name = get_nicest_module_name()
        return name

    def set_default_format(self, name=None):
        self.set_format(DEFAULT_FORMAT)

    def get_logger(self, name=None):
        name = name or self.current_logger_name()
        return logging.getLogger(name)

    def get_handler(self, name=None, top_level=True):
        name = name or self.current_logger_name()
        if top_level:
            name = name.split(".")[0]
        logger = self.get_logger(name)
        for h in logger.handlers:
            if isinstance(h, (StdHandler, logging.NullHandler)):
                return h

        raise KeyError("no logx-related handler found on this logger")

    def set_format(self, formatstring, logger_name=None, top_level=True, datefmt=None):
        logger_name = logger_name or self.current_logger_name()
        if top_level:
            logger_name = logger_name.split(".")[0]
        try:
            handler = self.get_handler(logger_name)
        except KeyError:
            handler = None
        if not handler:
            handler = self.add_std_handler(logger_name)
        if datefmt:
            formatter = logging.Formatter(formatstring, datefmt=datefmt)
        else:
            formatter = logging.Formatter(formatstring)
        handler.setFormatter(formatter)

    def set_null_handler(self, name=None, top_level=True):
        name = name or self.current_logger_name()
        if top_level:
            name = name.split(".")[0]
        logger = self.get_logger(name=name)
        logger.handlers = [_ for _ in logger.handlers if not isinstance(_, StdHandler)]
        logger.addHandler(logging.NullHandler())

    def clear_null_handler(self, name=None, top_level=True):
        name = name or self.current_logger_name()
        if top_level:
            name = name.split(".")[0]
        logger = self.get_logger(name=name)
        logger.handlers = [
            _ for _ in logger.handlers if not isinstance(_, logging.NullHandler)
        ]

    def get_logger_with_handler(self, logger_name):
        """
        Returns the form of logger_name, potentially converted in to top
        level, that has a handler

        Args:
            logger_name (str): the expected name of the logger

        Return:
            str: The name of the logger that has a handler

        Raises:
            KeyError: When it can't find a logger with a handler by that name
                in any form
        """
        try:
            self.get_handler(logger_name)
        except KeyError:
            logger_name = logger_name.split(".")[0]
            self.get_handler(logger_name)
        return logger_name

    def __getattr__(self, method, *args, **kwargs):
        if method.startswith("_"):
            raise AttributeError

        logger_name = self.current_logger_name()
        try:
            logger_name = self.get_logger_with_handler(logger_name)
        except KeyError:
            self.add_std_handler(logger_name)
        _logger = logging.getLogger(logger_name)
        return getattr(_logger, method)



import inspect


def get_calling_module(name=None, top_level_only=True):
    def top_module(mname):
        if top_level_only:
            return mname.split(".")[0]

        else:
            return mname

    s = inspect.stack()

    def modules_iter():
        for i in range(len(s)):
            m = inspect.getmodule(s[i][0])
            # func_name = s[i][3]
            yield m

    x = next(
        (
            m
            for m in modules_iter()
            if m and top_module(m.__name__) != top_module(__name__)
        ),
        None,
    )
    return x


def get_nicest_module_name(m=None):
    if not m:
        m = get_calling_module()
    if m:
        _name = m.__name__
    else:
        return "__main__"

    def is_main(name):
        return name == "__main__"

    if is_main(_name):
        if m.__spec__:
            return m.__spec__.name

        else:
            return inspect.getmodulename(inspect.getfile(m))

    else:
        return _name



import io
import os
import yaml

from logx import log


log.set_null_handler("logxutil")

ENV_YAML = ".env.yaml"


def load_env_yaml(path=None):
    path = path or ENV_YAML

    try:
        with io.open(path) as r:
            log.info(f"loading env vars from yaml: {path}")
            env_vars = yaml.load(r, Loader=yaml.FullLoader)
            os.environ.update(env_vars)
    except FileNotFoundError:
        log.info(f"local env file not found at {path}, not loading")


def env(name):
    return os.environ[name]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['logx', 'logxutil']

package_data = \
{'': ['*']}

install_requires = \
['logging_tree', 'pyyaml']

setup_kwargs = {
    'name': 'logx',
    'version': '0.1.1579232358',
    'description': 'best practice python logging with zero config',
    'long_description': '# logx: best practice python logging with zero config\n\nConfiguring logging is tedious. Reading the logging module docs makes me tired.\n\nWouldn\'t it be nice to log as easily as doing a print statement, without any upfront config?\n\n## Obligatory example\n\nEnter `logx`. It\'s as simple as:\n\n    >>> from logx import log\n    >>> log.info(\'hello world\')\n    hello world\n    >>> log.set_default_format()\n    >>> log.warn(\'warning!\')\n    [2018-02-26 21:51:16,971] WARNING [__main__.<module>:1] warning!\n\nLogs get logged automatically to the logger whose name matches the current module.\n\n## List of sweet features\n\n- Creates loggers lazily/as needed/on demand and **logs to the appropriate logger automatically**. If you\'re in the "acme" module it\'ll log to a log called "acme", no need worry about logger names and instances.\n- **Shows all log messages by default**, which follows the principle of least surprise and is probably what you want when debugging.\n- Included default handler **logs to the appropriate standard output stream by default**: Errors and warnings to stderr, the rest to stdout.\n- Allows easy following of best practice when including log statements in a library: **Just call log.create_null_handler() in your module.**\n- **Uses the standard logging library**, so you can still customize your setup as much as you want/need. Plays nicely with your existing logging config files.\n- **Includes the very useful logging_tree module** for viewing your current logging configuration. `logx.print_diagram()`\n\n## Install\n\n    >>> pip install logx\n\n## Contribute\n\nIssues and pull requests welcome, hit me. Am I doing logging completely wrong? Critique welcome, even if very pedantic.\n',
    'author': 'Robert Lechte',
    'author_email': 'robertlechte@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/djrobstep/logx',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
}


setup(**setup_kwargs)
